@@ -645,6 +645,27 @@ class Sf:
         results = results[['Object', 'RecordTypeName', 'Field_API', 'Controlling Field', 'defaultValueLabel', 'defaultValueValue', 'label', 'value', 'validFor']]
         return results
 
+
+
+    async def object_permission_bulk_check(self, objects: list, permissions: list ):
+        async_object_permission_check = async_wrap(self.object_permission_check)
+        result_list = await asyncio.gather(*[async_object_permission_check(object, permission ) for object in objects for permission in permissions])
+        # for result in result_list:
+        #     data = deep_merge_dictionaries(data, result)
+        result_list = [item for sublist in result_list for item in sublist]
+        return result_list
+
+    async def field_permission_bulk_check(self, objects: list, ) -> dict[str, pd.DataFrame]:
+        async_field_permission_check = async_wrap(self.field_permission_check)
+        result_list = await asyncio.gather(*[async_field_permission_check( object ) for object in objects])
+        # for result in result_list:
+        #     data = deep_merge_dictionaries(data, result)
+        profile_list = [item['profilePermissions'] for item in result_list]
+        permissionSet_list = [item['permissionSetPermissions'] for item in result_list]
+        result_dict = {'profilePermissions': pd.concat(profile_list), 'permissionSetPermissions': pd.concat(permissionSet_list)}
+        
+        return result_dict
+    
     ### ----------------------------------------------------------------------------------------------------
     ### Synchronous Metadata Functions
     ### ----------------------------------------------------------------------------------------------------
@@ -826,11 +847,69 @@ class Sf:
         
         Extra Notes and Examples of Usage
         """
-        sfRTDF = pd.DataFrame(self.sf.query("SELECT "+ ",".join(fieldAPINames) + " from RecordType WHERE SobjectType = '" + objectAPIName + "'")['records'])
+        sfRTDF = pd.DataFrame(self.sf.query_all("SELECT "+ ",".join(fieldAPINames) + " from RecordType WHERE SobjectType = '" + objectAPIName + "'")['records'])
         sfRTDF.drop(columns="attributes", inplace = True)
         return sfRTDF
 
-    
+    def object_permission_check(self, object, permission):
+        data = transform_sf_result_set_rec(self.sf.query_all(f"""SELECT Assignee.Id, Assignee.Name,Assignee.IsActive, Assignee.UserName, PermissionSet.Id, PermissionSet.isOwnedByProfile, PermissionSet.Profile.Name, PermissionSet.Label
+            FROM PermissionSetAssignment
+            WHERE PermissionSetId
+            IN (SELECT ParentId
+            FROM ObjectPermissions
+            WHERE SObjectType IN ('{object}') AND
+            ({permission} = true)) AND Assignee.IsActive = TRUE""")['records'])
+        
+        profile_violations = data[data['PermissionSet.IsOwnedByProfile']== True][['User.Username', 'Profile.Name']]
+        permission_set_violations = data[data['PermissionSet.IsOwnedByProfile']== False][['User.Username', 'PermissionSet.Label']]
+        result = []
+        for user in profile_violations['User.Username'].unique():
+            result.append({"UserName": user, 
+                            "Object API Name" : object,
+                            "Permission": permission,
+                            "Profile": "; ".join(list(profile_violations[profile_violations['User.Username'] == user]['Profile.Name'].unique())),
+                            "Permission Set": ""
+                        
+                        }
+                        
+                        )
+            
+        for user in permission_set_violations['User.Username'].unique():
+            result.append({"UserName": user, 
+                                                            "Object API Name" : object,
+                                                            "Permission": permission,
+                                                            "Profile": "",
+                                                            "Permission Set": "; ".join(list(permission_set_violations[permission_set_violations['User.Username'] == user]['PermissionSet.Label'].unique()))})
+        return result
+
+    def field_permission_check(self, object) -> dict[str, pd.DataFrame]:
+        fls_results = transform_sf_result_set_rec(self.sf.query_all(f"""
+                                            SELECT Field,ParentId,PermissionsEdit,PermissionsRead,SobjectType FROM FieldPermissions
+                                            WHERE SobjectType in ('{object}')
+                                            """)['records'])
+
+        permissionSetProfiles = transform_sf_result_set_rec(self.sf.query_all("""
+                                                    SELECT Id,Description,IsOwnedByProfile,Name,ProfileId, Profile.Name, Profile.Description,Type FROM PermissionSet
+                                                    """)['records'])
+
+        profiles = permissionSetProfiles[permissionSetProfiles['PermissionSet.IsOwnedByProfile'] == True][['PermissionSet.ProfileId','PermissionSet.Id','Profile.Name','Profile.Description']].copy()
+        permissionSets = permissionSetProfiles[permissionSetProfiles['PermissionSet.IsOwnedByProfile'] == False][['PermissionSet.Id','PermissionSet.Name','PermissionSet.Description']].copy()
+
+        permissionSetPermissions = pd.merge(fls_results, permissionSets, left_on='FieldPermissions.ParentId', right_on='PermissionSet.Id', how='inner').drop(columns=['PermissionSet.Id'], axis=1)
+        permissionSetPermissions.columns = ['Field API Name', 'Permission Set Id', 'FLS Edit', 'FLS Read', 'Object API Name', 'Permission Set Name', 'Permission Set Description']
+        permissionSetPermissions = permissionSetPermissions[['Permission Set Id', 'Permission Set Name', 'Permission Set Description', 'Object API Name', 'Field API Name', 'FLS Edit', 'FLS Read']]
+        permissionSetPermissions['Field API Name'] = permissionSetPermissions['Field API Name'].apply(lambda x: x.split('.')[-1])
+
+
+        profilePermissions = pd.merge(fls_results, profiles, left_on='FieldPermissions.ParentId', right_on='PermissionSet.Id', how='inner').drop(columns=['PermissionSet.Id','FieldPermissions.ParentId'], axis=1)
+        profilePermissions.columns = ['Field API Name', 'FLS Edit', 'FLS Read', 'Object API Name','Profile Id',  'Profile Name', 'Profile Description']
+        profilePermissions = profilePermissions[['Profile Id', 'Profile Name', 'Profile Description', 'Object API Name', 'Field API Name', 'FLS Edit', 'FLS Read']]
+        profilePermissions['Field API Name'] = profilePermissions['Field API Name'].apply(lambda x: x.split('.')[-1])
+        
+        return {'permissionSetPermissions': permissionSetPermissions, 'profilePermissions': profilePermissions}
+
+
+
     @serialze_zeep
     def metadata_read(self, metadata_type: str, fullName: str):
         return getattr(self.sf.mdapi, metadata_type).read(fullName)
